; records
;; All objects are meant to be read-only, but for explicit references
;;
;; We trust the program not to modify anything but this emulated type...


(provide 'record)
(require 'ref)
(require 'lib)
(require 'types)
(require 'object)


;(define

(define record:mark (types:make-mark 'record))
(define (new-record-type-definer)
   (let* ((field-count (ref:new 1))
         (field-list (deq:new))
	 (record-tag (cons record:mark field-count)))
   `((add-field
         ,(lambda (name . rest)
	      (let* ((fc (ref:++ field-count))
	          (getter (lambda (x)
              (dep:push! field-list `(,name ,fc ,@rest)))))
     (close
         ,(lambda ()
	      `((new
	           ,(lambda args
		        (let ((r (make-vector field-count record-tag)))
			    r)))
	        ,@(map
	        (get
		   ,(lambda


(define record-type::mark (types:make-mark 'record-type))

(define (ref:new . rest)
   "make a variable reference object,
that be initialized to point to object obj"
   (let ((obj (if (not (null? rest)) (car rest))))
   (cons ref::mark obj)))

(define (ref? o)
   (and (pair? o) (eq? (car o) ref::mark)))

(define (ref:get r)
   (ASSERT ref? r)
   ; make it atomic wrt GC
   (cdr r))

(define (ref:set! r v)
   (ASSERT ref? r)
   ; make it atomic wrt GC
   (set-cdr! r v))

(define (ref:map! r f)
   (ASSERT ref? r)
   ; make it atomic wrt GC
   (let ((v (cdr r)))
     (set-cdr! r (f v))
     v))

(define (ref:map! r f)
   "replaces the referenced element by its image by a given function"
   (ASSERT ref? r)
   ; make it atomic wrt GC
   (set-cdr! r (f (cdr r))))

(define (ref:postmap! r f)
   "replaces the referenced element by its image by a given function"
   "and returns the previous value"
   (ASSERT ref? r)
   ; make it atomic wrt GC
   (let ((v (cdr r)))
     (set-cdr! r (f v))
     v))

(define (ref:apply! r f . x)
   (ref:map! r (lambda (v) (apply f (cons v x)))))

(define (ref:postapply! r f . x)
   (ref:map! r (lambda (v) (apply f (cons v x)))))

(define (ref:++ r)
   (ref:postmap! r 1+))


(types:register 'ref ref?)

