#!/usr/local/bin/perl -w

# Computes Tunes dependencies.
#
# Currently supported:
# * provide and require or features in .scm files,
#  with at most one provided feature per file.
#  creates Makefile entries like
#	.PHONY:		feature
#	feature:	feature.scm requiredfeature1 requiredfeature2

use strict;


my $SCHEME_LOADPATH = ".:../generic:../../HLL/scm";
my @SCHEME_LOADPATH = split /:/, $SCHEME_LOADPATH;
my @FEATURE_LIST = ();

my $DEBUG = 0;
$| = $DEBUG;

sub errprint {
    print STDERR join(" ",@_)."\n";
}
sub db {
    if($DEBUG) {
	errprint @_;
    }
}

sub depend_file {
    my $file = shift;
    db "df: $file";
    open FILE, "<$file";
    my $provided = "";
    my @required = ();
    while(<FILE>) {
	if(/^\(provide *\'([^ ()]*)\)/i) {
	    db "detected provided feature $1 in $file...";
	    if($provided eq "") {
		$provided = $1.".foo";
		push @FEATURE_LIST, $provided;
	    } else {
		errprint
		    "File $file already provided $provided,",
		    "now provides $1!\n";
	    }
	};
	if(/\(require *\'([^ ()]*)/i) {
	    db "detected required feature $1 in $file...";
	    push @required, $1.".foo";
	};
    }
    db "dr: ",@required;
    if($provided eq "") {
	errprint "File $file does not provide anything!";
    } else {
	print "$provided:	".join(" ",$file,@required)."\n";
    }
    close FILE;
}

sub depend_dirs {
  my $dirname;
  while($dirname = shift) {
    opendir DIR, $dirname;
    my @sourcefiles = grep { /\.scm$/ } readdir(DIR);
    @sourcefiles = map { "$dirname/$_" } @sourcefiles;
    db "dd",@sourcefiles;
    map { depend_file($_) } @sourcefiles;
    closedir DIR;
  }
}

depend_dirs @SCHEME_LOADPATH;
print ".PHONY:	".join(" ",@FEATURE_LIST)."\n";
